import { activePossibleActions, alignmentActions, DEFAULT_CELL_INDENT, DEFAULT_FONT_SIZE, DEFAULT_FONT_SIZE_CHANGE_VALUE, INDENT_VALUE_CHANGE_VALUE, indentActions, instantActions, textBaseLineActions, } from "../data/constants.js";
import { ERibbonDataActions, ETextAlign, ETextBaseLine, } from "../data/enums.js";
// eventManager.ts
export class EventManager {
    constructor(excelsHandler, ribbon) {
        this.excelsHandler = excelsHandler;
        this.ribbon = ribbon;
        this.initializeElement();
        this.attachEvents();
    }
    get currentSheetObjHelper() {
        var _a;
        return (_a = this.excelsHandler.currSheetObj) === null || _a === void 0 ? void 0 : _a.instance.helper;
    }
    initializeElement() {
        //ribbon elements initalization
        this.tabButtons = document.querySelectorAll(".rumblesheet .tablist-items");
        this.toggleContentBtn = document.querySelector(".rumblesheet #toggle-content");
        this.focusZone = document.querySelector(".rumblesheet .focus-zone");
        this.featureMenuBtns = document.querySelectorAll(".rumblesheet .feature-menu");
        this.scrollLeftBtn = document.querySelector(".rumblesheet .scroll-left");
        this.scrollRightBtn = document.querySelector(".rumblesheet .scroll-right");
    }
    attachEvents() {
        this.attachRibbonEvents();
    }
    //* Attach Events Functions
    //ribbon specifc events
    attachRibbonTabEvents() {
        this.tabButtons.forEach((tab) => {
            tab.addEventListener("click", (e) => this.ribbon.ribbonFunctions.toggleTabForTarget(e.target));
        });
    }
    attachRibbonExpandedEvents() {
        if (this.toggleContentBtn) {
            this.toggleContentBtn.addEventListener("click", () => this.ribbon.ribbonFunctions.toggleContentBtnFunc());
        }
    }
    attachRibbonMenuButtonEvents() {
        this.featureMenuBtns.forEach((btn) => {
            btn.addEventListener("click", (e) => this.ribbon.ribbonFunctions.handleFeatureMenuToggle(e));
        });
    }
    attachRibbonScrollButtonEvents() {
        if (this.scrollLeftBtn && this.scrollRightBtn) {
            this.scrollLeftBtn.addEventListener("click", (e) => this.ribbon.ribbonFunctions.handleScrollRibbon(e));
            this.scrollRightBtn.addEventListener("click", (e) => this.ribbon.ribbonFunctions.handleScrollRibbon(e));
        }
    }
    attachRibbonFeatureSelectEvents() {
        const selectElementSelector = ".ribbon .select-options";
        document.querySelectorAll(selectElementSelector).forEach((selectEle) => {
            const ele = selectEle;
            if (ele) {
                ele.addEventListener("change", () => {
                    this.handleSelectChange(ele);
                });
            }
        });
    }
    attachRibbonFeatureColorEvents() {
        const chooseColorSelector = "input[type='color']";
        document.querySelectorAll(chooseColorSelector).forEach((colorEle) => {
            const ele = colorEle;
            if (ele) {
                ele.addEventListener("input", () => {
                    this.handleColorElement(ele);
                });
            }
        });
    }
    attachRibbonFeatureButtonEvents() {
        if (!this.focusZone) {
            //no focus zone detected
            return;
        }
        const selector = '.feature-box .feature-icon[role="button"], .feature-box .feature-horizontal[role="button"], .feature-box .feature-vertical[role="button"]';
        this.focusZone.addEventListener("click", (event) => {
            const target = event.target;
            const button = target.closest(selector);
            if (button) {
                this.performRibbonAction(button);
            }
        });
        // Keyboard event listener for accessibility
        this.focusZone.addEventListener("keydown", (event) => {
            const target = event.target;
            const button = target.closest(selector);
            if (button && (event.key === "Enter" || event.key === " ")) {
                event.preventDefault(); // Prevent scrolling when Space is pressed
                this.performRibbonAction(button);
            }
        });
    }
    attachRibbonFeatureEvents() {
        this.attachRibbonFeatureSelectEvents();
        this.attachRibbonFeatureColorEvents();
        this.attachRibbonFeatureButtonEvents();
    }
    //all events
    attachRibbonEvents() {
        this.attachRibbonTabEvents();
        this.attachRibbonExpandedEvents();
        this.attachRibbonMenuButtonEvents();
        this.attachRibbonScrollButtonEvents();
        this.attachRibbonFeatureEvents();
    }
    //* Handler Functions
    performRibbonAction(ele) {
        if (!ele) {
            return;
        }
        const action = ele.dataset.action;
        // Handling toggle actions (Bold, Italic, Underline)
        if (activePossibleActions.includes(action)) {
            ele.classList.toggle("active");
            const updatedState = ele.classList.contains("active");
            this.updateCellFormatting(action, updatedState);
        }
        // Handling actions like alignment
        else if (alignmentActions.includes(action)) {
            this.handleGroupRibbonActions(alignmentActions, ele);
            this.updateCellFormatting(action, true);
        }
        else if (textBaseLineActions.includes(action)) {
            this.handleGroupRibbonActions(textBaseLineActions, ele);
            this.updateCellFormatting(action, true);
        }
        // Handling indent actions
        else if (indentActions.includes(action)) {
            this.handleGroupRibbonActions(indentActions, ele);
            this.updateCellFormatting(action, true);
        }
        else if (instantActions.includes(action)) {
            this.updateCellFormatting(action, true);
        }
        else {
            console.log("Unhandled action: ", action);
        }
    }
    handleGroupRibbonActions(group, currEle) {
        group.forEach((groupAction) => {
            const indiEle = document.querySelector(`[data-action="${groupAction}"]`);
            if (indiEle)
                indiEle.classList.remove("active");
        });
        currEle.classList.add("active");
    }
    updateCellFormatting(action, isActive, newValue = "") {
        var _a, _b;
        console.log(action, isActive);
        const selectedCells = (_a = this.currentSheetObjHelper) === null || _a === void 0 ? void 0 : _a.mainCellManager.getCurrSelectedCells();
        if (selectedCells) {
            selectedCells.forEach((cellDetails) => {
                var _a, _b, _c, _d;
                if (cellDetails && cellDetails.cell) {
                    switch (action) {
                        case ERibbonDataActions.bold:
                            cellDetails.cell.styles.bold = isActive;
                            break;
                        case ERibbonDataActions.italic:
                            cellDetails.cell.styles.italic = isActive;
                            break;
                        case ERibbonDataActions.underline:
                            cellDetails.cell.styles.underline = isActive;
                            break;
                        case ERibbonDataActions.alignLeft:
                            cellDetails.cell.styles.textAlign = ETextAlign.left;
                            break;
                        case ERibbonDataActions.alignRight:
                            cellDetails.cell.styles.textAlign = ETextAlign.right;
                            break;
                        case ERibbonDataActions.alignCenter:
                            cellDetails.cell.styles.textAlign = ETextAlign.center;
                            break;
                        case ERibbonDataActions.increaseFont:
                            cellDetails.cell.styles.fontSize =
                                ((_a = cellDetails.cell.styles.fontSize) !== null && _a !== void 0 ? _a : DEFAULT_FONT_SIZE) +
                                    DEFAULT_FONT_SIZE_CHANGE_VALUE;
                            break;
                        case ERibbonDataActions.decreaseFont:
                            cellDetails.cell.styles.fontSize =
                                ((_b = cellDetails.cell.styles.fontSize) !== null && _b !== void 0 ? _b : DEFAULT_FONT_SIZE) -
                                    DEFAULT_FONT_SIZE_CHANGE_VALUE;
                            break;
                        case ERibbonDataActions.textBaselineTop:
                            cellDetails.cell.styles.textBaseline = ETextBaseLine.top;
                            console.log("done");
                            break;
                        case ERibbonDataActions.textBaselineMiddle:
                            cellDetails.cell.styles.textBaseline = ETextBaseLine.middle;
                            break;
                        case ERibbonDataActions.textBaselineBottom:
                            cellDetails.cell.styles.textBaseline = ETextBaseLine.bottom;
                            break;
                        case ERibbonDataActions.cut:
                            //todo - to be handled
                            break;
                        case ERibbonDataActions.copy:
                            //todo - to be handled
                            break;
                        case ERibbonDataActions.paste:
                            //todo - to be handled
                            break;
                        case ERibbonDataActions.fontFamily:
                            cellDetails.cell.styles.fontFamily = newValue;
                            break;
                        case ERibbonDataActions.fontSize:
                            cellDetails.cell.styles.fontSize = parseInt(newValue);
                            break;
                        case ERibbonDataActions.fillColor:
                            cellDetails.cell.styles.fill = newValue;
                            break;
                        case ERibbonDataActions.textColor:
                            cellDetails.cell.styles.color = newValue;
                            break;
                        case ERibbonDataActions.increaseIndent:
                            //todo - handle properly after resize is implemented
                            cellDetails.cell.styles.textIndent =
                                ((_c = cellDetails.cell.styles.textIndent) !== null && _c !== void 0 ? _c : DEFAULT_CELL_INDENT) +
                                    INDENT_VALUE_CHANGE_VALUE;
                            break;
                        case ERibbonDataActions.decreaseIndent:
                            cellDetails.cell.styles.textIndent = Math.max(0, ((_d = cellDetails.cell.styles.textIndent) !== null && _d !== void 0 ? _d : DEFAULT_CELL_INDENT) -
                                INDENT_VALUE_CHANGE_VALUE);
                            break;
                        default:
                            console.log("other button clicked");
                            break;
                    }
                }
            });
        }
        (_b = this.currentSheetObjHelper) === null || _b === void 0 ? void 0 : _b.mainCellManager.draw();
    }
    handleSelectChange(selectElement) {
        const ele = selectElement;
        const action = ele.dataset.action;
        if (ele && ele.value && action) {
            this.updateCellFormatting(action, true, ele.value);
        }
    }
    handleColorElement(inputTypeColorEle) {
        const ele = inputTypeColorEle;
        const action = ele.dataset.action;
        if (ele && ele.value && action) {
            this.updateCellFormatting(action, true, ele.value);
        }
    }
}
