import { DEFAULT_CELL_STYLES } from "./constants.js";
export class Cell {
    constructor(rowValue, colValue, value, nextRow, nextCol, prevRow, prevCol, styles) {
        this.rowValue = rowValue;
        this.colValue = colValue;
        this.value = value;
        this.nextRow = nextRow;
        this.nextCol = nextCol;
        this.prevRow = prevRow;
        this.prevCol = prevCol;
        this.styles = Object.assign(Object.assign({}, DEFAULT_CELL_STYLES), styles); // Ensures `styles` is never undefined
    }
}
export class SparseMatrix {
    constructor() {
        this.rowHeaders = {};
        this.colHeaders = {};
    }
    _cellExists(row, col) {
        let current = this.rowHeaders[row];
        while (current) {
            if (current.colValue === col)
                return true;
            current = current.nextCol;
        }
        return false;
    }
    _shiftRow(row, newRow) {
        let current = this.rowHeaders[row];
        while (current) {
            current.rowValue = newRow;
            current = current.nextCol;
        }
        this.rowHeaders[newRow] = this.rowHeaders[row];
        delete this.rowHeaders[row];
    }
    _shiftColumn(col, newCol) {
        let current = this.colHeaders[col];
        while (current) {
            current.colValue = newCol;
            current = current.nextRow;
        }
        this.colHeaders[newCol] = this.colHeaders[col];
        delete this.colHeaders[col];
    }
    _insertCellInRow(row, newCell) {
        let current = this.rowHeaders[row];
        let prev = undefined;
        while (current && current.colValue < newCell.colValue) {
            prev = current;
            current = current.nextCol;
        }
        if (prev) {
            prev.nextCol = newCell;
            newCell.prevCol = prev;
        }
        else {
            this.rowHeaders[row] = newCell;
        }
        if (current) {
            newCell.nextCol = current;
            current.prevCol = newCell;
        }
    }
    _insertCellInColumn(col, newCell) {
        let current = this.colHeaders[col];
        let prev = undefined;
        while (current && current.rowValue < newCell.rowValue) {
            prev = current;
            current = current.nextRow;
        }
        if (prev) {
            prev.nextRow = newCell;
            newCell.prevRow = prev;
        }
        else {
            this.colHeaders[col] = newCell;
        }
        if (current) {
            newCell.nextRow = current;
            current.prevRow = newCell;
        }
    }
    _shiftCellsRight(row, col) {
        let current = this.rowHeaders[row];
        while (current && current.colValue < col)
            current = current.nextCol;
        while (current) {
            this._shiftColumn(current.colValue, current.colValue + 1);
            current = current.nextCol;
        }
    }
    _shiftCellsDown(row, col) {
        let current = this.colHeaders[col];
        while (current && current.rowValue < row)
            current = current.nextRow;
        while (current) {
            this._shiftRow(current.rowValue, current.rowValue + 1);
            current = current.nextRow;
        }
    }
    _removeCellFromRow(row, col) {
        let current = this.rowHeaders[row];
        let prev = undefined;
        while (current && current.colValue !== col) {
            prev = current;
            current = current.nextCol;
        }
        if (!current)
            return;
        if (prev) {
            prev.nextCol = current.nextCol;
        }
        else {
            this.rowHeaders[row] = current.nextCol;
        }
        if (current.nextCol) {
            current.nextCol.prevCol = prev;
        }
    }
    _removeCellFromColumn(row, col) {
        let current = this.colHeaders[col];
        let prev = undefined;
        while (current && current.rowValue !== row) {
            prev = current;
            current = current.nextRow;
        }
        if (!current)
            return;
        if (prev) {
            prev.nextRow = current.nextRow;
        }
        else {
            this.colHeaders[col] = current.nextRow;
        }
        if (current.nextRow) {
            current.nextRow.prevRow = prev;
        }
    }
    createCell(row, col, value) {
        if (this._cellExists(row, col))
            return;
        const newNode = new Cell(row, col, value);
        if (!this.rowHeaders[row]) {
            this.rowHeaders[row] = newNode;
        }
        else {
            this._insertCellInRow(row, newNode);
        }
        if (!this.colHeaders[col]) {
            this.colHeaders[col] = newNode;
        }
        else {
            this._insertCellInColumn(col, newNode);
        }
    }
    _updateCellValue(row, col, value) {
        let current = this.rowHeaders[row];
        while (current) {
            if (current.colValue === col) {
                current.value = value;
                return;
            }
            current = current.nextCol;
        }
    }
    addRowInBetween(newRow) {
        Object.keys(this.rowHeaders)
            .map(Number)
            .sort((a, b) => b - a)
            .forEach(row => {
            if (row >= newRow)
                this._shiftRow(row, row + 1);
        });
        for (let col in this.colHeaders) {
            const newCell = new Cell(newRow, parseInt(col), null);
            this._insertCellInColumn(parseInt(col), newCell);
        }
    }
    addColumnInBetween(newCol) {
        Object.keys(this.colHeaders)
            .map(Number)
            .sort((a, b) => b - a)
            .forEach(col => {
            if (col >= newCol)
                this._shiftColumn(col, col + 1);
        });
        for (let row in this.rowHeaders) {
            const newCell = new Cell(parseInt(row), newCol, null);
            this._insertCellInRow(parseInt(row), newCell);
            this._insertCellInColumn(newCol, newCell);
        }
    }
    deleteRow(rowToDelete) {
        let current = this.rowHeaders[rowToDelete];
        while (current) {
            this._removeCellFromColumn(current.colValue, rowToDelete);
            current = current.nextCol;
        }
        delete this.rowHeaders[rowToDelete];
        Object.keys(this.rowHeaders)
            .map(Number)
            .sort((a, b) => a - b)
            .forEach(row => {
            if (row > rowToDelete)
                this._shiftRow(row, row - 1);
        });
    }
    deleteColumn(colToDelete) {
        let current = this.colHeaders[colToDelete];
        while (current) {
            this._removeCellFromRow(current.rowValue, colToDelete);
            current = current.nextRow;
        }
        delete this.colHeaders[colToDelete];
        Object.keys(this.colHeaders)
            .map(Number)
            .sort((a, b) => a - b)
            .forEach(col => {
            if (col > colToDelete)
                this._shiftColumn(col, col - 1);
        });
    }
    getCell(row, col) {
        let current = this.rowHeaders[row];
        while (current) {
            if (current.colValue === col)
                return current;
            current = current.nextCol;
        }
        return null;
    }
    setCell(row, col, value) {
        if (this._cellExists(row, col)) {
            this._updateCellValue(row, col, value);
        }
        else {
            this.createCell(row, col, value);
        }
    }
}
