import { ECanvasType } from "../../data/enums.js";
import { Cell, SparseMatrix } from "../../data/sparseMatrix.js";
import { GridHeaderManager } from "../controllers/gridManager.js";
import { Scroll } from "../controllers/scroll.js"; // Assuming Scroll is imported from scroll.ts
import { SheetRendrer } from "../controllers/sheetRendrer.js";
export class Helper {
    constructor(Sheet) {
        this.minZoom = 0.1;
        this.maxZoom = 5;
        this.loadedRows = 0;
        this.loadedCols = 0;
        this.sheet = Sheet;
        this.totalRow = 100;
        this.totalCol = 60;
        this.canvases = {};
        this.contexts = {};
        this.zoomIndex = 1;
        this.initCanvases();
        this.SparseMatrix = new SparseMatrix();
        this.GridHeaderManager = new GridHeaderManager(this, this.canvases.spreadsheet.clientWidth, this.canvases.spreadsheet.clientHeight, this.zoomIndex, this.totalRow, this.totalCol);
        this.scroll = new Scroll(this);
        this.sheetRendrer = new SheetRendrer(this);
        this.scroll.setRenderer(this.sheetRendrer);
        this.mainCellManager = this.sheet.mainCellManager;
    }
    initCanvases() {
        Object.values(ECanvasType).forEach((type) => {
            const canvas = document.getElementById(`${type}Canvas_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`);
            if (!canvas) {
                throw new Error(`Canvas not found: ${type}Canvas_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`);
            }
            this.canvases[type] = canvas;
            this.contexts[type] = canvas.getContext("2d");
        });
        this.verticalScroll = {
            scroll: document.getElementById(`verticalScroll_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`),
            bar: document.getElementById(`verticalBar_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`),
        };
        this.horizontalScroll = {
            scroll: document.getElementById(`horizontalScroll_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`),
            bar: document.getElementById(`horizontalBar_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`),
        };
    }
    getCanvases() {
        return this.canvases;
    }
    getContexts() {
        return this.contexts;
    }
    loadMoreContent(direction) {
        var _a;
        return (_a = this.GridHeaderManager) === null || _a === void 0 ? void 0 : _a.loadMoreContent(direction);
    }
    getScrollbarRatio(direction) {
        var _a;
        return (_a = this.GridHeaderManager) === null || _a === void 0 ? void 0 : _a.getScrollbarRatio(direction);
    }
    numberToColumnName(num) {
        let columnName = "";
        while (num > 0) {
            num--;
            columnName = String.fromCharCode(65 + (num % 26)) + columnName;
            num = Math.floor(num / 26);
        }
        return columnName;
    }
    letterToNumber(letter) {
        return letter
            .split("")
            .reduce((number, char) => number * 26 + (char.charCodeAt(0) - "A".charCodeAt(0) + 1), 0);
    }
    getCell(x, y) {
        return this.SparseMatrix.getCell(x, y);
    }
    getRowColofExcel() {
        return {
            row: this.sheet.row,
            col: this.sheet.col,
            index: this.sheet.index,
        };
    }
    updateZoomIndex(viewportWidth, viewportHeight, zoomIndex) {
        var _a;
        this.zoomIndex = zoomIndex;
        (_a = this.GridHeaderManager) === null || _a === void 0 ? void 0 : _a.update(viewportWidth, viewportHeight, this.zoomIndex);
    }
    updateCells() {
        var _a;
        (_a = this.GridHeaderManager) === null || _a === void 0 ? void 0 : _a.generateNewCells();
    }
    getRowheader() {
        return this.SparseMatrix.rowHeaders;
    }
    setValue(rows) {
        rows.forEach((row, rowIndex) => {
            row.forEach((cellValue, colIndex) => {
                if (cellValue !== null && cellValue !== undefined && cellValue !== "") {
                    // Ensure the row and column exist in SparseMatrix
                    this.SparseMatrix.addRowInBetween(rowIndex);
                    this.SparseMatrix.addColumnInBetween(colIndex);
                    // Get the cell at rowIndex
                    let existingCell = this.SparseMatrix.rowHeaders[rowIndex];
                    // If no cell exists in the row, create it
                    if (!existingCell) {
                        existingCell = new Cell(rowIndex, colIndex, cellValue);
                        this.SparseMatrix.rowHeaders[rowIndex] = existingCell;
                    }
                    // Check if the cell exists in the current row for this column
                    let currentCell = existingCell;
                    while (currentCell) {
                        if (currentCell.colValue === colIndex) {
                            // If the cell exists, update its value
                            currentCell.value = cellValue;
                            break;
                        }
                        currentCell = currentCell.nextCol; // Move to the next column in the row
                    }
                    // If the column does not exist, add a new cell for the column in this row
                    if (!currentCell) {
                        const newCell = new Cell(rowIndex, colIndex, cellValue);
                        this.SparseMatrix.addColumnInBetween(rowIndex);
                        this.SparseMatrix.addRowInBetween(colIndex);
                    }
                }
            });
        });
    }
    updateDrawForFeatures() {
        if (this.sheet.mainCellManager) {
            this.sheet.mainCellManager.updateDrawForScrolling();
        }
    }
    getScrollRatio(direction) {
        // Determine the ID of the scroll element based on the direction
        const scrollElementId = direction === "horizontal"
            ? `horizontalScroll_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`
            : `verticalScroll_${this.sheet.row}_${this.sheet.col}_${this.sheet.index}`;
        // Get the scroll element by its ID
        const scrollElement = document.getElementById(scrollElementId);
        // Check if the element exists
        if (!scrollElement) {
            throw new Error(`${direction} scroll element not found.`);
        }
        // Ensure the element is of type HTMLElement
        if (!(scrollElement instanceof HTMLElement)) {
            throw new Error(`${direction} scroll element is not a valid HTMLElement.`);
        }
        // Determine maxScroll based on the direction
        const maxScroll = direction === "horizontal"
            ? this.scroll.maxScrollX
            : this.scroll.maxScrollY;
        // Get the relevant dimension based on direction
        const clientSize = direction === "horizontal"
            ? scrollElement.clientWidth
            : scrollElement.clientHeight;
        // Calculate and return the scroll ratio
        return clientSize / (clientSize + maxScroll);
    }
    getScroll() {
        return this.scroll.getScroll();
    }
    // Customize scroll
    setScroll(x, y) {
        this.scroll.setScroll(x, y);
    }
    // update the max scroll by getting new width and height from grid header manager
    updateMaxScroll() {
        const totalWidth = this.GridHeaderManager.getTotalContainerWidth();
        const totalHeight = this.GridHeaderManager.getTotalContainerHeight();
        const visibleWidth = this.canvases.spreadsheet.clientWidth;
        const visibleHeight = this.canvases.spreadsheet.clientHeight;
        this.scroll.updateMaxScroll(totalWidth, totalHeight, visibleWidth, visibleHeight);
    }
    getTotalWidth() {
        return this.GridHeaderManager.getTotalContainerWidth();
    }
    getTotalHeight() {
        return this.GridHeaderManager.getTotalContainerHeight();
    }
    //Get Horizontal header cells
    getHorizontalHeaderCells(scrollX) {
        return this.GridHeaderManager.getHeaderCellsHorizontal(scrollX);
    }
    // Get Vertical header cells
    getVerticalHeaderCells(scrollY) {
        return this.GridHeaderManager.getHeaderCellsVertical(scrollY);
    }
    draw() {
        this.sheetRendrer.draw();
    }
    binarySearch(cells, value, property) {
        let low = 0;
        let high = cells.length - 1;
        while (low <= high) {
            const mid = Math.floor((low + high) / 2);
            const cell = cells[mid];
            const x = property === "x" ? cell.x : cell.y;
            if (x <= value &&
                value < x + (property === "x" ? cell.width : cell.height)) {
                return mid;
            }
            else if (x > value) {
                high = mid - 1;
            }
            else {
                low = mid + 1;
            }
        }
        return low;
    }
    setCell(rowNumber, columnNumber, value) {
        this.SparseMatrix.setCell(rowNumber, columnNumber, value);
    }
}
