import { DEFAULT_CANVAS_LINE_WIDTH, DEFAULT_CANVAS_TEXT_ALIGN, DEFAULT_CANVAS_TEXT_BASELINE, DEFAULT_CANVAS_LINES_COLOR, DEFAULT_CANVAS_TEXT_COLOR, DEFAULT_CANVAS_FONT_FAMILY, DEFAULT_MIN_PADDING_IN_CELL, DEFAULT_CELL_BG_COLOR, DEFAULT_CELL_FONT_COLOR, DEFAULT_FONT_SIZE } from "../../data/constants.js";
import { ETextAlign, ETextBaseLine } from "../../data/enums.js";
export class SheetRendrer {
    constructor(helper) {
        this.lastVisibleRow = 0;
        this.lastVisibleCol = 0;
        this.helper = helper;
        this.zoomIndex = this.helper.zoomIndex;
        this.minZoom = this.helper.minZoom;
        this.maxZoom = this.helper.maxZoom;
        this.baseGridSize = 20;
        this.lastDevicePixelRatio = window.devicePixelRatio;
        this.mappingFromHelper();
        this.setUpResizeObserver();
        this.setupEventListeners();
        this.monitorDevicePixelRatio();
    }
    mappingFromHelper() {
        this.resizeObserver = this.helper.resizeObserver;
        this.canvases = this.helper.canvases;
        this.contexts = this.helper.contexts;
    }
    setUpResizeObserver() {
        // Set up the ResizeObserver
        this.resizeObserver = new ResizeObserver(this.handleResize.bind(this));
        this.resizeObserver.observe(this.canvases.spreadsheet);
    }
    resizeCanvases() {
        var _a;
        const dpr = window.devicePixelRatio;
        Object.values((_a = this.canvases) !== null && _a !== void 0 ? _a : {}).forEach((canvas) => this.updateCanvasDimensions(canvas !== null && canvas !== void 0 ? canvas : {}, dpr));
        this.updateHeaderCells();
        this.draw();
        this.helper.updateDrawForFeatures();
    }
    updateCanvasDimensions(canvas, dpr) {
        const rect = canvas.getBoundingClientRect();
        canvas.width = rect.width * dpr;
        canvas.height = rect.height * dpr;
        const ctx = canvas.getContext("2d");
        ctx.scale(dpr, dpr);
    }
    setupEventListeners() {
        window.addEventListener("resize", this.handleResize.bind(this));
        this.canvases.spreadsheet.addEventListener("wheel", this.handleWheel.bind(this), { passive: false });
    }
    handleResize() {
        this.resizeCanvases();
    }
    handleWheel(event) {
        if (event.ctrlKey || event.metaKey) {
            event.preventDefault();
            const delta = event.deltaY;
            const zoomDelta = delta > 0 ? 0.9 : 1.1;
            const rect = this.canvases.spreadsheet.getBoundingClientRect();
            const mouseX = event.clientX - rect.left;
            const mouseY = event.clientY - rect.top;
            this.zoom(zoomDelta, mouseX, mouseY);
        }
    }
    zoom(zoomDelta, centerX, centerY) {
        const buffer = 5;
        const newScale = Math.min(Math.max(this.zoomIndex * zoomDelta, this.minZoom), this.maxZoom);
        if (this.helper.totalCol === 0 && this.helper.totalRow === 0) {
            this.applyzoom(newScale);
        }
        else if ((this.lastVisibleRow + buffer < this.helper.totalRow && this.lastVisibleCol + buffer < this.helper.totalCol) ||
            newScale > this.zoomIndex) {
            this.applyzoom(newScale);
        }
        else if ((this.helper.totalCol === 0 && this.lastVisibleRow + buffer < this.helper.totalRow) || newScale > this.zoomIndex) {
            this.applyzoom(newScale);
        }
        else if ((this.helper.totalRow === 0 && this.lastVisibleCol + buffer < this.helper.totalCol) || newScale > this.zoomIndex) {
            this.applyzoom(newScale);
        }
    }
    applyzoom(zoomIndex) {
        this.zoomIndex = zoomIndex;
        this.updateHeaderCells();
        this.helper.updateMaxScroll();
        this.draw();
        this.helper.updateDrawForFeatures();
    }
    monitorDevicePixelRatio() {
        const checkDevicePixelRatio = () => {
            const currentDevicePixelRatio = window.devicePixelRatio;
            if (currentDevicePixelRatio !== this.lastDevicePixelRatio) {
                this.lastDevicePixelRatio = currentDevicePixelRatio;
                this.resizeCanvases();
            }
            requestAnimationFrame(checkDevicePixelRatio);
        };
        requestAnimationFrame(checkDevicePixelRatio);
    }
    clearCanvases() {
        var _a;
        Object.entries((_a = this.contexts) !== null && _a !== void 0 ? _a : {}).forEach(([type, ctx]) => {
            const canvas = this.canvases[type];
            ctx.clearRect(0, 0, canvas.width / window.devicePixelRatio, canvas.height / window.devicePixelRatio);
        });
    }
    updateHeaderCells() {
        const visibleWidth = this.canvases.spreadsheet.clientWidth;
        const visibleHeight = this.canvases.spreadsheet.clientHeight;
        this.helper.updateZoomIndex(visibleWidth, visibleHeight, this.zoomIndex);
        this.helper.updateMaxScroll();
    }
    draw() {
        this.clearCanvases();
        // Get the scroll values
        const { x: scrollX, y: scrollY } = this.helper.getScroll();
        // Check if more content needs to be loaded
        this.drawHeaders(scrollX, scrollY);
        this.drawGrid(scrollX, scrollY);
        this.drawSparseMatrixValues(scrollX, scrollY);
    }
    drawHeaders(scrollX, scrollY) {
        this.verticalCells = this.helper.getVerticalHeaderCells(scrollY);
        this.lastVisibleRow = this.verticalCells[this.verticalCells.length - 1].row;
        this.horizontalCells = this.helper.getHorizontalHeaderCells(scrollX);
        this.lastVisibleCol = this.horizontalCells[this.horizontalCells.length - 1].col;
        this.drawHeaderCells(this.contexts.vertical, this.verticalCells, true, scrollY);
        this.drawHeaderCells(this.contexts.horizontal, this.horizontalCells, false, scrollX);
    }
    drawHeaderCells(ctx, cells, isVertical, scroll) {
        ctx.lineWidth = DEFAULT_CANVAS_LINE_WIDTH;
        ctx.textAlign = DEFAULT_CANVAS_TEXT_ALIGN;
        ctx.textBaseline = DEFAULT_CANVAS_TEXT_BASELINE;
        ctx.strokeStyle = DEFAULT_CANVAS_LINES_COLOR;
        ctx.fillStyle = DEFAULT_CANVAS_TEXT_COLOR;
        const canvasWidth = this.canvases[isVertical ? "vertical" : "horizontal"].width / window.devicePixelRatio;
        const canvasHeight = this.canvases[isVertical ? "vertical" : "horizontal"].height / window.devicePixelRatio;
        cells.forEach((cell) => {
            const drawCell = (isVertical &&
                cell.y - scroll < canvasHeight &&
                cell.y + cell.height - scroll > 0) ||
                (!isVertical &&
                    cell.x - scroll < canvasWidth &&
                    cell.x + cell.width - scroll > 0);
            if (drawCell) {
                ctx.beginPath();
                if (isVertical) {
                    const y = cell.y - scroll;
                    ctx.moveTo(0, y);
                    ctx.lineTo(canvasWidth, y);
                    ctx.stroke();
                    this.drawAtCentered(ctx, cell.value.toString(), canvasWidth / 2, y + cell.height / 2, canvasWidth, cell.height);
                }
                else {
                    const x = cell.x - scroll;
                    ctx.moveTo(x, 0);
                    ctx.lineTo(x, canvasHeight);
                    ctx.stroke();
                    this.drawAtCentered(ctx, cell.value.toString(), x + cell.width / 2, canvasHeight / 2, cell.width, canvasHeight);
                }
            }
        });
    }
    drawAtCentered(ctx, text, x, y, maxWidth, maxHeight) {
        const baseFontSize = this.baseGridSize * this.zoomIndex;
        let fontSize = Math.min(baseFontSize, maxHeight * 0.8);
        // Adjust font size if text is too wide
        // ctx.font = `${fontSize}px Arial`;
        let textWidth = ctx.measureText(text).width;
        if (textWidth > maxWidth * 0.9) {
            fontSize *= (maxWidth * 0.9) / textWidth;
        }
        ctx.font = `${fontSize}px ${DEFAULT_CANVAS_FONT_FAMILY}`;
        ctx.strokeStyle = DEFAULT_CANVAS_LINES_COLOR;
        ctx.fillText(text, x, y, maxWidth);
    }
    drawGrid(scrollX, scrollY) {
        const ctx = this.contexts.spreadsheet;
        const verticalCells = this.verticalCells;
        const horizontalCells = this.horizontalCells;
        ctx.strokeStyle = DEFAULT_CANVAS_LINES_COLOR;
        ctx.lineWidth = DEFAULT_CANVAS_LINE_WIDTH;
        const canvasWidth = this.canvases.spreadsheet.width / window.devicePixelRatio;
        const canvasHeight = this.canvases.spreadsheet.height / window.devicePixelRatio;
        verticalCells.forEach((cell) => {
            const y = cell.y - scrollY;
            ctx.beginPath();
            ctx.moveTo(0, y);
            ctx.lineTo(canvasWidth, y);
            ctx.stroke();
        });
        horizontalCells.forEach((cell) => {
            const x = cell.x - scrollX;
            ctx.beginPath();
            ctx.moveTo(x, 0);
            ctx.lineTo(x, canvasHeight);
            ctx.stroke();
        });
    }
    drawSparseMatrixValues(scrollX, scrollY) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
        const ctx = this.contexts.spreadsheet;
        const visibleWidth = this.canvases.spreadsheet.width / window.devicePixelRatio;
        const visibleHeight = this.canvases.spreadsheet.height / window.devicePixelRatio;
        // Map vertical and horizontal cells for faster lookup
        const verticalCellMap = new Map(this.verticalCells.map((cell) => [cell.value, cell]));
        const horizontalCellMap = new Map(this.horizontalCells.map((cell) => [cell.value, cell]));
        // Iterate through the sparse matrix rows
        // ------------------------- need to fix a better approach is there ----------------------
        let rowHeader = this.helper.getRowheader();
        for (const row in rowHeader) {
            let current = rowHeader[row];
            while (current) {
                // Find corresponding header cells
                const vCell = verticalCellMap.get(current.rowValue);
                const hCell = horizontalCellMap.get(this.helper.numberToColumnName(current.colValue));
                // If both header cells are found (i.e., the cell is visible)
                if (vCell && hCell) {
                    const cellX = hCell.x - scrollX;
                    const cellY = vCell.y - scrollY;
                    // Only render cells within the visible area
                    if (cellX < visibleWidth &&
                        cellY < visibleHeight &&
                        cellX + hCell.width > 0 &&
                        cellY + vCell.height > 0) {
                        // Save context and set text styles
                        ctx.save();
                        if (current) {
                            ctx.fillStyle = (_a = current.styles.color) !== null && _a !== void 0 ? _a : DEFAULT_CANVAS_TEXT_COLOR;
                            let fontSize = ((_c = (_b = current.styles) === null || _b === void 0 ? void 0 : _b.fontSize) !== null && _c !== void 0 ? _c : DEFAULT_FONT_SIZE) * this.zoomIndex;
                            let fontFamily = ((_e = (_d = current.styles) === null || _d === void 0 ? void 0 : _d.fontFamily) !== null && _e !== void 0 ? _e : DEFAULT_CANVAS_FONT_FAMILY);
                            let fontWeight = ((_f = current.styles) === null || _f === void 0 ? void 0 : _f.bold) ? "bold" : "normal";
                            let fontStyle = ((_g = current.styles) === null || _g === void 0 ? void 0 : _g.italic) ? "italic" : "normal";
                            ctx.font = `${fontStyle} ${fontWeight} ${fontSize}px ${fontFamily}`;
                        }
                        // Clip the rendering area to the cell's rectangle
                        ctx.beginPath();
                        ctx.fillStyle = (_h = current.styles.fill) !== null && _h !== void 0 ? _h : DEFAULT_CELL_BG_COLOR;
                        ctx.fillRect(cellX + DEFAULT_CANVAS_LINE_WIDTH / 2, cellY + DEFAULT_CANVAS_LINE_WIDTH / 2, hCell.width - DEFAULT_CANVAS_LINE_WIDTH, vCell.height - DEFAULT_CANVAS_LINE_WIDTH); //todo shiv why i need to do this.
                        ctx.fillStyle = (_j = current.styles.color) !== null && _j !== void 0 ? _j : DEFAULT_CELL_FONT_COLOR;
                        ctx.rect(cellX, cellY, hCell.width, vCell.height);
                        ctx.clip();
                        //todo make seperate funtion for handle text align and baseline
                        // Draw the text if it exists
                        if (current.value !== undefined && current.value !== null) {
                            //default X and Y positon of text in cell
                            let textX = cellX + ((_k = current.styles.textIndent) !== null && _k !== void 0 ? _k : 0) + hCell.width / 2;
                            let textY = cellY + vCell.height / 2;
                            ctx.textAlign = DEFAULT_CANVAS_TEXT_ALIGN;
                            if (current.styles.textAlign === ETextAlign.left) {
                                textX = cellX + ((_l = current.styles.textIndent) !== null && _l !== void 0 ? _l : 0) + DEFAULT_MIN_PADDING_IN_CELL;
                                ctx.textAlign = ETextAlign.left;
                            }
                            else if (current.styles.textAlign === ETextAlign.center) {
                                textX = cellX + ((_m = current.styles.textIndent) !== null && _m !== void 0 ? _m : 0) + hCell.width / 2;
                                ctx.textAlign = ETextAlign.center;
                            }
                            else if (current.styles.textAlign === ETextAlign.right) {
                                textX = cellX + ((_o = current.styles.textIndent) !== null && _o !== void 0 ? _o : 0) + hCell.width - DEFAULT_MIN_PADDING_IN_CELL;
                                ctx.textAlign = ETextAlign.right;
                            }
                            if (current.styles.textBaseline === ETextBaseLine.top) {
                                textY = cellY + DEFAULT_MIN_PADDING_IN_CELL;
                                ctx.textBaseline = ETextBaseLine.top;
                            }
                            else if (current.styles.textBaseline === ETextBaseLine.middle) {
                                textY = cellY + vCell.height / 2;
                                ctx.textBaseline = ETextBaseLine.middle;
                            }
                            else if (current.styles.textBaseline === ETextBaseLine.bottom) {
                                textY = cellY + vCell.height - DEFAULT_MIN_PADDING_IN_CELL;
                                ctx.textBaseline = ETextBaseLine.bottom;
                            }
                            // Draw centered text in the cell
                            ctx.fillText(current.value.toString(), textX, textY);
                        }
                        // Restore the context state after rendering the cell
                        ctx.restore();
                    }
                }
                // Move to the next column in the current row
                current = current.nextCol;
            }
        }
    }
}
