export class Scroll {
    constructor(helper) {
        this.helper = helper;
        this.scrollX = 0;
        this.scrollY = 0;
        this.maxScrollX = 0;
        this.maxScrollY = 0;
        this.isDragging = false;
        this.isScrollbarDragging = false;
        this.lastMouseX = 0;
        this.lastMouseY = 0;
        this.loadThreshold = 0.8;
        this.helper = helper;
        this.mappingFromHelper();
        this.setupEventListeners();
    }
    mappingFromHelper() {
        this.canvases = this.helper.canvases;
        this.contexts = this.helper.contexts;
    }
    setupEventListeners() {
        const canvas = this.canvases.spreadsheet;
        const { row: Sheetrow, col: Sheetcol, index: Sheetindex } = this.helper.getRowColofExcel();
        canvas.addEventListener('wheel', this.handleWheel.bind(this), { passive: false });
        canvas.addEventListener('mousedown', this.handleMouseDown.bind(this));
        document.addEventListener('mousemove', this.handleMouseMove.bind(this));
        document.addEventListener('mouseup', this.handleMouseUp.bind(this));
        const verticalScrollBar = document.getElementById(`verticalBar_${Sheetrow}_${Sheetcol}_${Sheetindex}`);
        const horizontalScrollBar = document.getElementById(`horizontalBar_${Sheetrow}_${Sheetcol}_${Sheetindex}`);
        verticalScrollBar === null || verticalScrollBar === void 0 ? void 0 : verticalScrollBar.addEventListener('mousedown', this.handleScrollBarMouseDown.bind(this, 'vertical'));
        horizontalScrollBar === null || horizontalScrollBar === void 0 ? void 0 : horizontalScrollBar.addEventListener('mousedown', this.handleScrollBarMouseDown.bind(this, 'horizontal'));
    }
    setRenderer(renderer) {
        this.sheetRendrer = renderer;
        this.setScrollLimits();
    }
    setScrollLimits() {
        const viewportWidth = this.canvases.spreadsheet.clientWidth;
        const viewportHeight = this.canvases.spreadsheet.clientHeight;
        const totalWidth = this.helper.getTotalWidth();
        const totalHeight = this.helper.getTotalHeight();
        this.updateMaxScroll(totalWidth, totalHeight, viewportWidth, viewportHeight);
    }
    updateMaxScroll(totalWidth, totalHeight, viewportWidth, viewportHeight) {
        this.maxScrollX = Math.max(0, totalWidth - viewportWidth);
        this.maxScrollY = Math.max(0, totalHeight - viewportHeight);
        // Ensure current scroll positions don't exceed new maximums
        this.scrollX = Math.min(this.scrollX, this.maxScrollX);
        this.scrollY = Math.min(this.scrollY, this.maxScrollY);
        this.updateScrollBar('horizontal');
        this.updateScrollBar('vertical');
    }
    updateScrollBar(direction) {
        var _a, _b, _c, _d;
        const scrollElement = direction === 'vertical'
            ? (_a = this.helper.verticalScroll) === null || _a === void 0 ? void 0 : _a.scroll
            : (_b = this.helper.horizontalScroll) === null || _b === void 0 ? void 0 : _b.scroll;
        const barElement = direction === 'vertical'
            ? (_c = this.helper.verticalScroll) === null || _c === void 0 ? void 0 : _c.bar
            : (_d = this.helper.horizontalScroll) === null || _d === void 0 ? void 0 : _d.bar;
        if (!scrollElement || !barElement || !this.helper) {
            return;
        }
        const minBarSize = 20;
        const scrollSize = direction === 'vertical' ? scrollElement.clientHeight : scrollElement.clientWidth;
        // Get ratio from GridHeaderManager
        const ratio = this.helper.getScrollbarRatio(direction);
        // Calculate bar size
        const barSize = Math.max(minBarSize, scrollSize * ratio);
        // Calculate position
        const maxScroll = direction === 'vertical' ? this.maxScrollY : this.maxScrollX;
        const currentScroll = direction === 'vertical' ? this.scrollY : this.scrollX;
        const scrollableSpace = scrollSize - barSize;
        const scrollProgress = maxScroll > 0 ? currentScroll / maxScroll : 0;
        const barPosition = scrollableSpace * scrollProgress;
        // Update bar style
        if (direction === 'vertical') {
            barElement.style.height = `${barSize}px`;
            barElement.style.top = `${barPosition}px`;
        }
        else {
            barElement.style.width = `${barSize}px`;
            barElement.style.left = `${barPosition}px`;
        }
    }
    setScroll(deltaX, deltaY) {
        const maxScrollSpeed = 100;
        deltaX = Math.max(-maxScrollSpeed, Math.min(deltaX, maxScrollSpeed));
        deltaY = Math.max(-maxScrollSpeed, Math.min(deltaY, maxScrollSpeed));
        // Calculate new scroll positions
        let newScrollX = Math.max(0, Math.min(this.scrollX + deltaX, this.maxScrollX));
        let newScrollY = Math.max(0, Math.min(this.scrollY + deltaY, this.maxScrollY));
        // Update scroll positions
        if (newScrollX !== this.scrollX || newScrollY !== this.scrollY) {
            this.scrollX = newScrollX;
            this.scrollY = newScrollY;
            if (deltaX !== 0)
                this.updateScrollBar('horizontal');
            if (deltaY !== 0)
                this.updateScrollBar('vertical');
            this.helper.draw();
        }
        // Check if we need to load more content
        const shouldLoadMore = this.checkAndLoadMoreContent(newScrollX, newScrollY);
        if (shouldLoadMore) {
            // Update scroll limits based on new content
            this.setScrollLimits();
        }
        this.helper.updateDrawForFeatures();
    }
    handleScrollBarMouseDown(direction, event) {
        event.preventDefault();
        this.isScrollbarDragging = true;
        this.scrollbarDirection = direction;
        this.lastMouseX = event.clientX;
        this.lastMouseY = event.clientY;
    }
    handleMouseMove(event) {
        if (this.isDragging) {
            if (event.shiftKey) {
                const deltaX = this.lastMouseX - event.clientX;
                const deltaY = this.lastMouseY - event.clientY;
                this.setScroll(deltaX, deltaY);
                this.lastMouseX = event.clientX;
                this.lastMouseY = event.clientY;
            }
        }
        else if (this.isScrollbarDragging && this.scrollbarDirection) {
            const delta = this.scrollbarDirection === 'vertical'
                ? event.clientY - this.lastMouseY
                : event.clientX - this.lastMouseX;
            const scrollRatio = this.scrollbarDirection === 'vertical'
                ? this.helper.getScrollRatio("vertical")
                : this.helper.getScrollRatio("horizontal");
            const scrollDelta = delta / scrollRatio;
            if (this.scrollbarDirection === 'vertical') {
                this.setScroll(0, scrollDelta);
            }
            else {
                this.setScroll(scrollDelta, 0);
            }
            this.lastMouseX = event.clientX;
            this.lastMouseY = event.clientY;
        }
    }
    handleMouseUp() {
        this.isDragging = false;
        this.isScrollbarDragging = false;
        this.destroy();
    }
    handleWheel(event) {
        if (!event.ctrlKey && !event.metaKey) {
            event.preventDefault();
            const deltaX = event.deltaX;
            const deltaY = event.deltaY;
            this.setScroll(deltaX, deltaY);
        }
    }
    handleMouseDown(event) {
        this.isDragging = true;
        this.lastMouseX = event.clientX;
        this.lastMouseY = event.clientY;
    }
    checkAndLoadMoreContent(newScrollX, newScrollY) {
        var _a, _b, _c, _d;
        let contentLoaded = false;
        if (!this.helper) {
            return contentLoaded;
        }
        // Check vertical scroll
        const verticalBarElement = (_a = this.helper.verticalScroll) === null || _a === void 0 ? void 0 : _a.bar;
        const verticalBarTop = (verticalBarElement === null || verticalBarElement === void 0 ? void 0 : verticalBarElement.offsetTop) || 0;
        const verticalScrollableHeight = (((_b = this.helper.verticalScroll) === null || _b === void 0 ? void 0 : _b.scroll.clientHeight) || 0) - ((verticalBarElement === null || verticalBarElement === void 0 ? void 0 : verticalBarElement.clientHeight) || 0);
        if (verticalScrollableHeight > 0 && (verticalBarTop / verticalScrollableHeight) >= this.loadThreshold) {
            if (this.helper.loadMoreContent('vertical')) {
                contentLoaded = true;
            }
        }
        // Check horizontal scroll
        const horizontalBarElement = (_c = this.helper.horizontalScroll) === null || _c === void 0 ? void 0 : _c.bar;
        const horizontalBarLeft = (horizontalBarElement === null || horizontalBarElement === void 0 ? void 0 : horizontalBarElement.offsetLeft) || 0;
        const horizontalScrollableWidth = (((_d = this.helper.horizontalScroll) === null || _d === void 0 ? void 0 : _d.scroll.clientWidth) || 0) - ((horizontalBarElement === null || horizontalBarElement === void 0 ? void 0 : horizontalBarElement.clientWidth) || 0);
        if (horizontalScrollableWidth > 0 && (horizontalBarLeft / horizontalScrollableWidth) >= this.loadThreshold) {
            if (this.helper.loadMoreContent('horizontal')) {
                contentLoaded = true;
            }
        }
        return contentLoaded;
    }
    // Remove expandContent and checkScrollPosition methods as they're no longer needed
    constrainScrollPosition(scrollValue, direction) {
        const maxScroll = direction === "horizontal" ? this.maxScrollX : this.maxScrollY;
        return Math.max(0, Math.min(scrollValue, maxScroll));
    }
    getScroll() {
        return { x: this.scrollX, y: this.scrollY };
    }
    destroy() {
        const canvas = this.canvases.spreadsheet;
        canvas.removeEventListener('wheel', this.handleWheel);
        canvas.removeEventListener('mousedown', this.handleMouseDown);
        document.removeEventListener('mousemove', this.handleMouseMove);
        document.removeEventListener('mouseup', this.handleMouseUp);
    }
}
