import { DEFAULT_HIGHLIGHT_BORDER_COLOR, DEFAULT_HIGHLIGHT_FILL_COLOR, DEFAULT_HIGHLIGHT_HEADER_BORDER_COLOR, DEFAULT_HIGHLIGHT_HEADER_FILL_COLOR, DEFAULT_HIGHLIGHT_HEADER_LINE_WIDTH, DEFAULT_HIGHLIGHT_LINE_WIDTH, DEFAULT_HORIZONTAL_CANVAS_HEIGHT, DEFAULT_VERTICAL_CANVAS_WIDTH, } from "../../../data/constants.js";
export class Selection {
    constructor(maincellManager) {
        this.maincellManager = maincellManager;
        this.canvases = this.maincellManager.getCanvases();
        this.selectedCells = [];
        this.isDragging = false;
        this.isScrolling = false;
        this.setupEventListeners();
    }
    setupEventListeners() {
        const canvas = this.canvases.spreadsheet;
        canvas.addEventListener("pointerdown", this.handlePointerDown.bind(this));
        document.addEventListener("pointerup", this.handlePointerUp.bind(this));
        document.addEventListener("pointermove", this.handlePointerMove.bind(this));
    }
    handlePointerDown(event) {
        event.preventDefault();
        this.startPoint = this.maincellManager.getCanvasCoordinates(event);
        this.handleCellClick(event);
        if (!this.isDragging) {
            this.selectedCells = [];
            this.isDragging = true;
            this.isScrolling = true;
            this.updateSelectedCells(this.startPoint);
        }
    }
    handlePointerUp() {
        this.isScrolling = false;
        this.removeEventListeners();
        if (this.isDragging) {
            this.isDragging = false;
        }
    }
    handlePointerMove(event) {
        if (this.isDragging) {
            const currentPoint = this.maincellManager.getCanvasCoordinates(event);
            this.updateSelectedCells(currentPoint);
            this.handleScrolling(event);
        }
    }
    handleScrolling(event) {
        const { x, y } = this.maincellManager.getCanvasCoordinates(event);
        const { x: scrollX, y: scrollY } = this.maincellManager.getScroll();
        // Check if the pointer is near the edges to trigger scrolling
        const edgeDistance = 30; // Distance from edge to start scrolling
        const canvas = this.canvases.spreadsheet;
        if (x - scrollX < 0 && event.movementX < 0) {
            this.maincellManager.setScroll(-10, 0);
        }
        else if (x - scrollX > canvas.clientWidth - edgeDistance &&
            event.movementX > 0) {
            this.maincellManager.setScroll(10, 0);
        }
        if (y - scrollY < 0 && event.movementY < 0) {
            this.maincellManager.setScroll(0, -10); // Scroll up
        }
        else if (y - scrollY > canvas.clientHeight - edgeDistance &&
            event.movementY > 0) {
            this.maincellManager.setScroll(0, 10); // Scroll down
        }
        this.maincellManager.updateInputElement(this.clickedCell_headercells);
    }
    handleCellClick(event) {
        if (this.isDragging)
            return; // Prevent handling cell click if dragging
        const { x: scrollX, y: scrollY } = this.maincellManager.getCanvasCoordinates(event);
        this.clickedCell_headercells = this.maincellManager.getCellFromCoordinates(scrollX, scrollY);
        if (this.clickedCell_headercells) {
            this.deselectCurrentCells();
            // this.maincellManager.updateInputElement(this.clickedCell_headercells);
            this.selectCell(this.clickedCell_headercells);
            this.updateSelectedCells(this.startPoint);
            // this.drawHighlight();
        }
    }
    selectCell(cell) {
        this.maincellManager.updateInputElement(cell);
        this.maincellManager.draw();
        // this.drawHighlight();
    }
    deselectCurrentCells() {
        if (this.selectedCells.length > 0) {
            this.selectedCells = [];
            this.maincellManager.hideInputElement(); // Hide the input element if it's visible
            this.maincellManager.draw(); // Redraw the sheet to remove any highlighting
        }
    }
    updateDrawForScrolling() {
        this.maincellManager.updateInputElement(this.clickedCell_headercells);
        this.selectCell(this.clickedCell_headercells);
        this.drawHighlight();
    }
    updateSelectedCells(endPoint) {
        const cells = this.maincellManager.getCellsFromRect(this.startPoint, endPoint);
        this.selectedCells = cells;
        this.drawHighlight();
    }
    drawHighlight() {
        this.maincellManager.draw();
        const ctx = this.canvases.spreadsheet.getContext("2d");
        const { x: scrollX, y: scrollY } = this.maincellManager.getScroll();
        if (this.selectedCells.length === 0)
            return;
        let width = null;
        let height = null;
        ctx.strokeStyle = DEFAULT_HIGHLIGHT_BORDER_COLOR;
        ctx.lineWidth = DEFAULT_HIGHLIGHT_LINE_WIDTH;
        // Get the boundary of the selected area
        const minX = Math.min(...this.selectedCells.map((cell) => cell.column.x));
        const maxX = Math.max(...this.selectedCells.map((cell) => {
            width = cell.column.width;
            return cell.column.x + cell.column.width;
        }));
        const minY = Math.min(...this.selectedCells.map((cell) => cell.row.y));
        const maxY = Math.max(...this.selectedCells.map((cell) => {
            height = cell.row.height;
            return cell.row.y + cell.row.height;
        }));
        // Use a single function to draw on both horizontal and vertical canvases
        this.drawRectangleOnHeaderCanvas("horizontal", minX - scrollX, 0, maxX - minX, height, true);
        this.drawRectangleOnHeaderCanvas("vertical", 0, minY - scrollY, width, maxY - minY, false);
        // Draw the border around the entire selection area
        ctx.fillStyle = DEFAULT_HIGHLIGHT_FILL_COLOR;
        ctx.fillRect(minX - scrollX, minY - scrollY, maxX - minX, maxY - minY);
        ctx.strokeRect(minX - scrollX, minY - scrollY, maxX - minX, maxY - minY);
    }
    drawRectangleOnHeaderCanvas(type, x, y, width = 0, height = 0, isHorizontal) {
        const context = this.maincellManager.getContexts();
        const ctx = context[type];
        ctx.beginPath();
        ctx.fillStyle = DEFAULT_HIGHLIGHT_HEADER_FILL_COLOR;
        ctx.fillRect(x, y, width, height);
        if (isHorizontal) {
            // Draw a solid green line at the bottom
            ctx.strokeStyle = DEFAULT_HIGHLIGHT_HEADER_BORDER_COLOR;
            ctx.lineWidth = DEFAULT_HIGHLIGHT_HEADER_LINE_WIDTH;
            ctx.moveTo(x, DEFAULT_HORIZONTAL_CANVAS_HEIGHT);
            ctx.lineTo(x + width, DEFAULT_HORIZONTAL_CANVAS_HEIGHT);
        }
        else {
            // Draw a solid green line on the right side
            ctx.strokeStyle = DEFAULT_HIGHLIGHT_HEADER_BORDER_COLOR;
            ctx.lineWidth = DEFAULT_HIGHLIGHT_HEADER_LINE_WIDTH;
            ctx.moveTo(DEFAULT_VERTICAL_CANVAS_WIDTH, y);
            ctx.lineTo(DEFAULT_VERTICAL_CANVAS_WIDTH, y + height);
        }
        ctx.stroke(); // Finalize the drawing
    }
    removeEventListeners() {
        const canvas = this.canvases.spreadsheet;
        canvas.removeEventListener("pointerdown", this.handlePointerDown);
        document.removeEventListener("pointerup", this.handlePointerUp);
        document.removeEventListener("pointermove", this.handlePointerMove);
    }
}
